@@ -90,6 +90,9 @@ class Simple:
     def attron(self, attr):
         self.screen.attron(attr)
 
+    def attroff(self, attr):
+        self.screen.attroff(attr)
+
     def clrtoeol(self):
         self.screen.clrtoeol()
 
@@ -223,6 +226,9 @@ class Columns:
     def attron(self, attr):
         self.attrs |= attr
 
+    def attroff(self, attr):
+        self.attrs &= ~attr
+
     def clrtoeol(self):
         self.curwin.clrtoeol()
 
@@ -299,35 +305,34 @@ simple_characters = bytearray(
         b'\xb4\xb6\xb7\xc3\xc4\xd6\xdc\xe4\xe9\xfc\xf6')
 
 class Terminal:
-    def __init__(self, acsc, columns, reverse=False):
+    def __init__(self, acsc, screenfactory):
         self.mode = (self.feed_simple,)
         self.realscreen = None
         self.screen = None
         self.fg = self.bg = 0
         self.graphics_font = False
-        self.graphics_chars = acsc # really initialized after
+        self.graphics_chars = acsc # really initialized in __enter__
         self.lastchar = ord(b' ')
-        self.columns = columns
-        self.reverse = reverse
+        self.screenfactory = screenfactory
         self.need_refresh = None
 
-    def switchmode(self):
-        if isinstance(self.screen, Columns):
+    def makescreen(self, switch=False):
+        try:
+            if isinstance(self.screen, Simple) ^ switch:
+                raise BadWidth("use simple screen")
+            self.screen = self.screenfactory(self.realscreen)
+        except BadWidth:
             self.screen = Simple(self.realscreen)
-        else:
-            self.screen = Columns(self.realscreen, self.columns)
         self.request_refresh()
 
+    def switchmode(self):
+        self.makescreen(True)
+
     def resized(self):
         # The refresh call causes curses to notice the new dimensions.
         self.realscreen.refresh()
         self.realscreen.clear()
-        try:
-            self.screen = Columns(self.realscreen, self.columns,
-                                  reverse=self.reverse)
-        except BadWidth:
-            self.screen = Simple(self.realscreen)
-        self.request_refresh()
+        self.makescreen()
 
     def resizepty(self, ptyfd):
         ym, xm = self.screen.getmaxyx()
@@ -360,8 +365,7 @@ class Terminal:
         self.realscreen.keypad(1)
         curses.start_color()
         init_color_pairs()
-        self.screen = Columns(self.realscreen, self.columns,
-                              reverse=self.reverse)
+        self.makescreen()
         curses.noecho()
         curses.raw()
         self.graphics_chars = compose_dicts(self.graphics_chars, acs_map())
@@ -516,6 +520,10 @@ class Terminal:
         self.screen.move(n - 1, x)
         self.request_refresh()
 
+    def do_u7(self):
+        y, x = self.screen.getyx()
+        return b"\x1b[%d;%dR" % (y + 1, x + 1)
+
     def feed_reset(self):
         if self.graphics_font:
             self.mode = (self.feed_graphics,)
@@ -523,7 +531,7 @@ class Terminal:
             self.mode = (self.feed_simple,)
 
     def feed(self, char):
-        self.mode[0](char, *self.mode[1:])
+        return self.mode[0](char, *self.mode[1:])
 
     feed_simple_table = {
         ord('\a'): do_bel,
@@ -603,15 +611,19 @@ class Terminal:
             self.screen.attrset(0)
         elif 30 <= code <= 37:
             self.fg = code - 30
+            self.screen.attroff(curses.A_COLOR)
             self.screen.attron(get_color(self.fg, self.bg))
         elif code == 39:
             self.fg = 7
+            self.screen.attroff(curses.A_COLOR)
             self.screen.attron(get_color(self.fg, self.bg))
         elif 40 <= code <= 47:
             self.bg = code - 40
+            self.screen.attroff(curses.A_COLOR)
             self.screen.attron(get_color(self.fg, self.bg))
         elif code == 49:
             self.bg = 0
+            self.screen.attroff(curses.A_COLOR)
             self.screen.attron(get_color(self.fg, self.bg))
         else:
             raise ValueError("feed esc [ %r m" % code)
@@ -653,6 +665,8 @@ class Terminal:
             self.request_refresh()
         elif char == ord(b'K') and prev == b'1':
             self.do_el1()
+        elif char == ord(b'n') and prev == b'6':
+            return self.do_u7()
         else:
             raise ValueError("feed esc [ %r %r" % (prev, char))
 
@@ -794,10 +808,13 @@ def main():
     options, args = parser.parse_args()
     keymapping, acsc = compute_keymap(symbolic_keymapping)
 
+    def screenfactory(realscreen):
+        return Columns(realscreen, options.columns, reverse=options.reverse)
+
     process = ForkPty(args or [os.environ["SHELL"]], dict(TERM="ansi"))
     try:
         with process as masterfd:
-            with Terminal(acsc, options.columns, reverse=options.reverse) as t:
+            with Terminal(acsc, screenfactory) as t:
                 t.resizepty(masterfd)
                 process.start()
                 while True:
@@ -834,13 +851,15 @@ def main():
                         if not data:
                             break
                         for char in bytearray(data):
-                            if "TCVT_DEVEL" in os.environ:
-                                t.feed(char)
+                            try:
+                                response = t.feed(char)
+                            except ValueError:
+                                if "TCVT_DEVEL" in os.environ:
+                                    raise
+                                t.feed_reset()
                             else:
-                                try:
-                                    t.feed(char)
-                                except ValueError:
-                                    t.feed_reset()
+                                if response:
+                                    os.write(masterfd, response)
                         t.refresh(0.1)
                     else:
                         t.refresh()
